package de.zimkand.contentproclient;

import android.Manifest;
import android.content.ContentResolver;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.database.Cursor;
import android.net.Uri;
import android.provider.ContactsContract;
import android.os.Bundle;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.widget.ListView;
import android.widget.SimpleCursorAdapter;
import android.widget.Toast;

public class MainActivity extends AppCompatActivity
        implements ActivityCompat.OnRequestPermissionsResultCallback {

    private final String TAG = "ContentProClient";
    private final int PERMISSIONS_REQUEST_READ_CONTACTS = 1;
    private final int PERMISSIONS_REQUEST_WRITE_CONTACTS = 2;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        // Nötigenfalls die WRITE-Berechtigung anfordern
        if (ContextCompat.checkSelfPermission(this,
                Manifest.permission.WRITE_CONTACTS)
                != PackageManager.PERMISSION_GRANTED)
            ActivityCompat.requestPermissions(this,
                    new String[]{Manifest.permission.WRITE_CONTACTS}, PERMISSIONS_REQUEST_WRITE_CONTACTS);
        else
            modifyContacts();

        // Nötigenfalls die READ-Berechtigung anfordern
        if (ContextCompat.checkSelfPermission(this,
                Manifest.permission.READ_CONTACTS)
                != PackageManager.PERMISSION_GRANTED)
            ActivityCompat.requestPermissions(this,
                    new String[]{Manifest.permission.READ_CONTACTS}, PERMISSIONS_REQUEST_READ_CONTACTS);
        else
            showContacts();
    }

    private void modifyContacts() {
        //  Quellen:
        // https://www.dev2qa.com/android-add-contact-programmatically-example/
        // https://developer.android.com/reference/android/provider/ContactsContract.RawContacts

        // Wir fügen zunächst einen leeren neuen Rohkontakt in die Tabelle ContactsContract.RawContacts des
        // Contacts-Providers ein und merken uns seine Kennung.

        ContentValues values = new ContentValues();
        Uri rawContactUri = getContentResolver().insert(ContactsContract.RawContacts.CONTENT_URI, values);
        // Get the newly created contacts raw id.
        long rawContactId = ContentUris.parseId(rawContactUri);

        // Nun wird der Anzeigename zum neuen Rohkontakt in die data-Tebelle des Contacts-Providers eingefügt.
        // Das ContentValues-Objekt zur neuen Zeile erhält zunächst für die Eigenschaft RAW_CONTACT_ID der
        // beim Einfügen in die Tabelle RawContacts zurückgemeldeten Wert:
        values.put(ContactsContract.Data.RAW_CONTACT_ID, rawContactId);
        // Eine Data-Tabellenzeile kann diverse Informationen aufnehmen (z.B. Anzeigename, Telefonnummern).
        // Dies ist der Typ für eine Namenszeile:
        // ContactsContract.CommonDataKinds.StructuredName.CONTENT_ITEM_TYPE
        // Diese Zeile hat 9 Spalten mit den Namen DATA1 bis DATA9, die jeweils zur Aufnahme einer bestimmten
        // Namensvariante vorgesehen sind. In der Spalte DATA1 mit dem Aliasnamen DISPLAY_NAME landet der
        // Anzeigename.
        values.put(ContactsContract.Data.MIMETYPE,
                   ContactsContract.CommonDataKinds.StructuredName.CONTENT_ITEM_TYPE);
        values.put(ContactsContract.CommonDataKinds.StructuredName.DISPLAY_NAME, "Kurt Saar");
        Uri result  = getContentResolver().insert(ContactsContract.Data.CONTENT_URI, values);
        if (result != null) {
            Toast.makeText(this, "Contact inserted", Toast.LENGTH_SHORT).show();
            Log.i(TAG, "Inserted ID: " + String.valueOf(rawContactId));
        }


        // Eintrag aktualisieren
        String where;
        String[] whereArgs;
        int rowsChanged = 0;
        values.clear();
        values.put(ContactsContract.CommonDataKinds.StructuredName.DISPLAY_NAME, "Ludger Saar");
        where = ContactsContract.Data.RAW_CONTACT_ID + " = ?";
        whereArgs = new String[] {String.valueOf(rawContactId)};
        rowsChanged = getContentResolver().update(ContactsContract.Data.CONTENT_URI, values, where, whereArgs);
        Toast.makeText(this, "Numer of rows changed: " + Integer.toString(rowsChanged), Toast.LENGTH_SHORT).show();


        // Ersten Eintrag mit einem bestimmten Anzeigenamen löschen
        // Zeile aus der Contacts-Tabelle Data löschen:
        rawContactId = getRawContactIdByName("Ludger", "Saar");
        where = ContactsContract.RawContacts._ID + " = ?";
        whereArgs = new String[] {String.valueOf(rawContactId)};
        Uri uri = ContactsContract.RawContacts.CONTENT_URI;
        uri.buildUpon().appendQueryParameter(ContactsContract.CALLER_IS_SYNCADAPTER, "false");
        rowsChanged = getContentResolver().delete(uri, where, whereArgs);
        Log.i(TAG, "Deleted from RawContacts table: " + String.valueOf(rowsChanged));
        Toast.makeText(this, "Numer of rows deleted: " + Integer.toString(rowsChanged), Toast.LENGTH_SHORT).show();
        // When a raw contact is deleted, a SQLite trigger deletes the parent contact.

        // SQL-Injektion
        String userInput = "' OR '1'='1";
        where = ContactsContract.RawContacts.DISPLAY_NAME_PRIMARY + "='"+userInput+"'";
        long anzahl = getContentResolver().delete(ContactsContract.RawContacts.CONTENT_URI, where, null);
        Log.i(TAG, "Deleted from RawContacts-table: " + String.valueOf(anzahl));

        // Der Contacts-Provider braucht eventuell etwas Zeit, bevor der neue Zustand abgefragt werden kann.
        try {Thread.sleep(500);} catch (Exception ex) {}
    }

    private void showContacts() {
        String[] projection = {ContactsContract.Contacts._ID,
                ContactsContract.Contacts.DISPLAY_NAME};
        Cursor cursor = null;
        try {
            cursor = getContentResolver().query(ContactsContract.Contacts.CONTENT_URI,
                    projection, null, null, null);
        } catch (Exception ex) {
            String message = ex.getMessage();
            Log.e(TAG, "Error: " + message, ex);
        }
        if (cursor != null){
            if (cursor.getCount() > 0 ) {
                int[] tvIds = {R.id.id, R.id.name};
                SimpleCursorAdapter adapter = new SimpleCursorAdapter(
                        getApplicationContext(),         // The application's Context object
                        R.layout.list_view_element,      // A layout in XML for one row in the ListView
                        cursor,                          // The result from the query
                        projection,                      // A string array of column names in the cursor
                        tvIds,                           // An integer array of view IDs in the row layout
                        0);                              // Flags (usually none are needed)
                ListView lv = findViewById(R.id.listView);
                lv.setAdapter(adapter);
            }
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode,
                                           String permissions[], int[] grantResults) {
        switch (requestCode) {
            case PERMISSIONS_REQUEST_WRITE_CONTACTS: {
                // If request is cancelled, the result arrays are empty.
                if (grantResults.length > 0
                        && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    modifyContacts();
                } else
                    Toast.makeText(this, "Write-Permission not granted", Toast.LENGTH_SHORT).show();
                return;
            }
            case PERMISSIONS_REQUEST_READ_CONTACTS: {
                // If request is cancelled, the result arrays are empty.
                if (grantResults.length > 0
                        && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    showContacts();
                } else
                    Toast.makeText(this, "Read-Permission not granted", Toast.LENGTH_SHORT).show();
                return;
            }
        }
    }

    // Die wesentlichen Ideen zur folgenden Methode stammen von: https://www.dev2qa.com/how-to-update-delete-android-contacts-programmatically/
    // Get raw contact id by contact given name and family name.
    // Return raw contact id.
    private long getRawContactIdByName(String givenName, String familyName) {
        ContentResolver contentResolver = getContentResolver();
        String[] projection = {ContactsContract.RawContacts._ID};
        String displayName = givenName + " " + familyName;
        String whereClause = ContactsContract.RawContacts.DISPLAY_NAME_PRIMARY + " = ?";
        String[] whereArgs = new String[] {displayName};

        Cursor cursor = contentResolver.query(ContactsContract.RawContacts.CONTENT_URI,
                projection, whereClause, whereArgs, null);
        long rawContactId = -1;
        if(cursor!=null) {
            int queryResultCount = cursor.getCount();
            Log.i(TAG, "Count of Query with Display Name: " + String.valueOf(queryResultCount));
            if(queryResultCount > 0) {
                cursor.moveToLast();
                rawContactId = cursor.getLong(cursor.getColumnIndex(ContactsContract.RawContacts._ID));
            }
        }
        return rawContactId;
    }
}
