package de.zimkand.contentproclient;

import android.Manifest;
import android.content.ContentResolver;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.pm.PackageManager;
import android.database.Cursor;
import android.net.Uri;
import android.provider.ContactsContract;
import android.os.Bundle;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.widget.ListView;
import android.widget.SimpleCursorAdapter;
import android.widget.Toast;

public class MainActivity extends AppCompatActivity
        implements ActivityCompat.OnRequestPermissionsResultCallback {

    private final String TAG = "ContentProClient";
    private final int PERMISSIONS_REQUEST_READ_CONTACTS = 1;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        // Nötigenfalls die READ-Berechtigung anfordern
        if (ContextCompat.checkSelfPermission(this,
                Manifest.permission.READ_CONTACTS)
                != PackageManager.PERMISSION_GRANTED)
            ActivityCompat.requestPermissions(this,
                    new String[]{Manifest.permission.READ_CONTACTS}, PERMISSIONS_REQUEST_READ_CONTACTS);
        else
            showContacts();
    }

    private void showContacts() {
        String[] projection = {ContactsContract.Contacts._ID, ContactsContract.Contacts.DISPLAY_NAME};
        Cursor cursor = getContentResolver().query(ContactsContract.Contacts.CONTENT_URI, projection, null, null, null);
        if (cursor != null){
            if (cursor.getCount() > 0 ) {
                int[] tvIds = {R.id.id, R.id.name};
                SimpleCursorAdapter adapter = new SimpleCursorAdapter(
                        getApplicationContext(),         // The application's Context object
                        R.layout.list_view_element,      // A layout in XML for one row in the ListView
                        cursor,                          // The result from the query
                        projection,                      // A string array of column names in the cursor
                        tvIds,                           // An integer array of view IDs in the row layout
                        0);                              // Flags (usually none are needed)
                ListView lv = findViewById(R.id.listView);
                lv.setAdapter(adapter);
            }
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode,
                                           String permissions[], int[] grantResults) {
        switch (requestCode) {
            case PERMISSIONS_REQUEST_READ_CONTACTS: {
                // If request is cancelled, the result arrays are empty.
                if (grantResults.length > 0
                        && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    showContacts();
                } else
                    Toast.makeText(this, "Read-Permission not granted", Toast.LENGTH_SHORT).show();
                return;
            }
        }
    }
}
