package de.zimkand.eventdiary;

import android.content.ContentUris;
import android.content.ContentValues;
import android.content.Intent;
import android.database.Cursor;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.ContextMenu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ListView;

import java.util.List;

public class EventListActivity extends AppCompatActivity implements AdapterView.OnItemClickListener {

    private final String TAG = this.getClass().getSimpleName();

    private Cursor cursor;
    private EventDiaryAdapter adapter;
    private ListView lv;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_event_list);

        lv = findViewById(R.id.listView);
        lv.setOnItemClickListener(this);
        registerForContextMenu(lv);

        cursor = getContentResolver().query(EventProvider.CONTENT_URI, null, null, null,
                                            EventProvider.COL_DATETIME + " DESC");

        if (cursor.getCount() == 0) {
            ContentValues values = new ContentValues();
            values.put(EventProvider.COL_DESC, "5 Cent gefunden");
            values.put(EventProvider.COL_MOOD, 5);
            values.put(EventProvider.COL_DATETIME, System.currentTimeMillis());
            getContentResolver().insert(EventProvider.CONTENT_URI, values);
            cursor.requery();
        }

        startManagingCursor(cursor);
        adapter = new EventDiaryAdapter(this, cursor);
        lv.setAdapter(adapter);
    }

    @Override
    public void onCreateContextMenu(ContextMenu contMenu, View v,
                                    ContextMenu.ContextMenuInfo contextMenuInfo) {
        super.onCreateContextMenu(contMenu, v, contextMenuInfo);
        getMenuInflater().inflate(R.menu.event_list_context_menu, contMenu);
    }

    @Override
    public boolean onContextItemSelected(MenuItem item) {
        AdapterView.AdapterContextMenuInfo acmi = (AdapterView.AdapterContextMenuInfo) item.getMenuInfo();
        switch (item.getItemId()) {
            case R.id.evliconit_edit:
                editEntry(acmi.position);
                return true;
            case R.id.evliconit_del:
                getContentResolver().delete(ContentUris.withAppendedId(EventProvider.CONTENT_URI, acmi.id), null, null);
                cursor.requery();
                return true;
            default:
                return super.onContextItemSelected(item);
        }
    }

    @Override
    public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
        editEntry(position);
    }

    private void editEntry(int position) {
        Cursor cursor = adapter.getCursor();
        cursor.moveToPosition(position);
        Intent intent = new Intent(this, EditEventActivity.class);
        intent.putExtra("rowid", cursor.getLong(0));
        intent.putExtra("desc", cursor.getString(2));
        intent.putExtra("mood", cursor.getInt(3));
        startActivity(intent);
    }
}
