package de.zimkand.eventdiary;

import android.content.ContentUris;
import android.content.ContentValues;
import android.content.Intent;
import android.database.Cursor;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.LoaderManager;
import android.support.v4.content.CursorLoader;
import android.support.v4.content.Loader;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.ContextMenu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ListView;

public class EventListActivity extends AppCompatActivity
        implements AdapterView.OnItemClickListener, LoaderManager.LoaderCallbacks<Cursor> {

    private final String TAG = this.getClass().getSimpleName();
    private final int REQUEST_CODE_EDIT = 1;
    private EventDiaryAdapter adapter;
    private ListView lv;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_event_list);

        lv = findViewById(R.id.listView);
        lv.setOnItemClickListener(this);
        registerForContextMenu(lv);

        adapter = new EventDiaryAdapter(this, null);
        lv.setAdapter(adapter);
        getSupportLoaderManager().initLoader(0, null, this);
    }

    @Override
    public void onCreateContextMenu(ContextMenu contMenu, View v,
                                    ContextMenu.ContextMenuInfo contextMenuInfo) {
        super.onCreateContextMenu(contMenu, v, contextMenuInfo);
        getMenuInflater().inflate(R.menu.event_list_context_menu, contMenu);
    }

    @Override
    public boolean onContextItemSelected(MenuItem item) {
        AdapterView.AdapterContextMenuInfo acmi = (AdapterView.AdapterContextMenuInfo) item.getMenuInfo();
        switch (item.getItemId()) {
            case R.id.evliconit_edit:
                editEntry(acmi.position);
                return true;
            case R.id.evliconit_del:
                int numDel = getContentResolver().delete(
                    ContentUris.withAppendedId(EventProvider.CONTENT_URI, acmi.id), null, null);
                if (numDel > 0)
                    getSupportLoaderManager().getLoader(0).onContentChanged();
                return true;
            default:
                return super.onContextItemSelected(item);
        }
    }

    @Override
    public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
        editEntry(position);
    }

    private void editEntry(int position) {
        Cursor cursor = adapter.getCursor();
        cursor.moveToPosition(position);
        Intent intent = new Intent(this, EditEventActivity.class);
        intent.putExtra("rowid", cursor.getLong(0));
        intent.putExtra("desc", cursor.getString(2));
        intent.putExtra("mood", cursor.getInt(3));
        startActivityForResult(intent, REQUEST_CODE_EDIT);
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent intent) {
        if (requestCode == REQUEST_CODE_EDIT)
            getSupportLoaderManager().getLoader(0).onContentChanged();
    }

    @NonNull
    @Override
    public Loader<Cursor> onCreateLoader(int id, @Nullable Bundle args) {
        return new CursorLoader(this,
                EventProvider.CONTENT_URI, null, null, null,
                EventProvider.COL_DATETIME + " DESC");
    }

    @Override
    public void onLoadFinished(@NonNull Loader<Cursor> loader, Cursor cursor) {
        if (cursor.getCount() == 0) {
            ContentValues values = new ContentValues();
            values.put(EventProvider.COL_DESC, "5 Cent gefunden");
            values.put(EventProvider.COL_MOOD, 5);
            values.put(EventProvider.COL_DATETIME, System.currentTimeMillis());
            getContentResolver().insert(EventProvider.CONTENT_URI, values);
            loader.onContentChanged();
        }
        adapter.swapCursor(cursor);
    }

    @Override
    public void onLoaderReset(@NonNull Loader<Cursor> loader) {
        adapter.swapCursor(null);
    }
}
