package de.zimkand.eventdiary;

import android.content.ContentProvider;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.UriMatcher;
import android.database.Cursor;
import android.net.Uri;
import android.util.Log;
import android.widget.Toast;

public class EventProvider extends ContentProvider {

    private static final String AUTHORITY = "de.zimkand.eventdiary.provider";
    public static final Uri CONTENT_URI = Uri.parse("content://"+AUTHORITY+"/"+ EventProvider.TABLE_EVENTS);
    public static final String TABLE_EVENTS = "events";
    public static final String COL_ID = "_id";
    public static final String COL_DATETIME = "datetime";
    public static final String COL_DESC = "desc";
    public static final String COL_MOOD = "mood";

    private EventDAO eventDAO;
    private static final UriMatcher uriMatcher;
    private static final int EVENTS_DIR = 1;
    private static final int EVENTS_ITEM = 2;

    static {
        uriMatcher = new UriMatcher(UriMatcher.NO_MATCH);
        uriMatcher.addURI(AUTHORITY, TABLE_EVENTS, EVENTS_DIR);
        uriMatcher.addURI(AUTHORITY, TABLE_EVENTS +"/#", EVENTS_ITEM);
    }

    @Override
    public boolean onCreate() {
        eventDAO = EventDatabase.getInstance(getContext()).getEventDAO();
        return true;
    }

    @Override
    public Cursor query(Uri uri, String[] projection, String selection,
                        String[] selectionArgs, String sortOrder) {
        Cursor result;
        switch (uriMatcher.match(uri)) {
            case EVENTS_DIR:
                result = eventDAO.selectAll();
                break;
            case EVENTS_ITEM:
                result = eventDAO.selectById(Long.parseLong(uri.getLastPathSegment()));
                break;
            default:
                throw new IllegalArgumentException("URI invalid: " + uri);
        }
        return result;
    }

    @Override
    public Uri insert(Uri uri, ContentValues values) {
        switch (uriMatcher.match(uri)) {
            case EVENTS_DIR:
                Event event = Event.fromContentValues(values);
                eventDAO.insert(event);
                Toast.makeText(getContext(), R.string.inserted, Toast.LENGTH_SHORT).show();
                return ContentUris.withAppendedId(uri, event.getId());
            case EVENTS_ITEM:
                throw new IllegalArgumentException("Insertion with ID not allowed: " + uri);
            default:
                throw new IllegalArgumentException("URI invalid: " + uri);
        }
    }

    @Override
    public int delete(Uri uri, String selection, String[] selectionArgs) {
        switch (uriMatcher.match(uri)) {
            case EVENTS_DIR:
                throw new IllegalArgumentException("Cannot delete without ID: " + uri);
            case EVENTS_ITEM: {
                int count = eventDAO.deleteById(ContentUris.parseId(uri));
                if (count > 0)
                    Toast.makeText(getContext(), R.string.deleted, Toast.LENGTH_SHORT).show();
                return count;
            }
            default:
                throw new IllegalArgumentException("URI invalid: " + uri);
        }
    }

    @Override
    public int update(Uri uri, ContentValues values, String selection, String[] selectionArgs) {
        switch (uriMatcher.match(uri)) {
            case EVENTS_DIR:
                throw new IllegalArgumentException("Cannot update without ID: " + uri);
            case EVENTS_ITEM: {
                Event event = eventDAO.getEvent(ContentUris.parseId(uri));
                event.setValues(values);
                int count = eventDAO.update(event);
                if (count > 0)
                    Toast.makeText(getContext(), R.string.updated, Toast.LENGTH_SHORT).show();
                return count;
                }
            default:
                throw new IllegalArgumentException("URI invalid: " + uri);
        }
    }

    @Override
    public String getType(Uri uri) {
        switch (uriMatcher.match(uri)) {
            case EVENTS_DIR:
                return "vnd.android.cursor.dir/vnd" + AUTHORITY + "." + TABLE_EVENTS;
            case EVENTS_ITEM:
                return "vnd.android.cursor.item/vnd" + AUTHORITY + "." + TABLE_EVENTS;
            default:
                throw new IllegalArgumentException("URI invalid: " + uri);
        }
    }
}
