/*
 * Copyright (C) 2010 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// Vorlage: https://developer.android.com/guide/components/fragments.html

package de.zimkand.fragmentdemo;

import android.support.v4.app.ListFragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.ListView;

public class TitlesFragment extends ListFragment implements FragmentManager.OnBackStackChangedListener {
    private boolean mDualPane;
    private int mCurCheckPosition;
    private FragmentManager fm;
    private boolean uselessStackState = true;

    @Override
    public void onActivityCreated(Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);

        fm = getFragmentManager();
        fm.addOnBackStackChangedListener(this);

        setListAdapter(new ArrayAdapter<String>(getActivity(),
                android.R.layout.simple_list_item_activated_1, getResources().getStringArray(R.array.titles)));

        mDualPane = getActivity().findViewById(R.id.details) != null;
        getListView().setChoiceMode(ListView.CHOICE_MODE_SINGLE);
        if (savedInstanceState != null)
            mCurCheckPosition = savedInstanceState.getInt("curChoice", 0);
        if (mDualPane) {
            if (savedInstanceState == null)
                getListView().setItemChecked(mCurCheckPosition, true);
            showDetails(mCurCheckPosition);
        }
    }

    @Override
    public void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        outState.putInt("curChoice", mCurCheckPosition);
    }

    @Override
    public void onListItemClick(ListView list, View v, int position, long id) {
        mCurCheckPosition = position;
        showDetails(position);
    }

    @Override
    public void onBackStackChanged() {
        DetailsFragment details = (DetailsFragment) fm.findFragmentById(R.id.details);
        getListView().setItemChecked(details.getShownIndex(), true);
    }

    private void showDetails(int index) {
        if (mDualPane) {
            DetailsFragment details = (DetailsFragment) fm.findFragmentById(R.id.details);
            // Tätig werden, wenn noch keine Details-Fragement vorhanden ist, oder wenn der falsche Text angezeigt wird.

            if (details == null)
                Log.i("Fragment", "Kein details-Fragment vorhanden");
            else
                Log.i("Fragment", "Vorhanden mit Index = "+ String.valueOf(details.getShownIndex()));

            if (details == null || details.getShownIndex() != index) {
                if (details != null)
                    Log.i("Fragment", "Neues Details-Fragment benötigt. Index="+String.valueOf(index) + ", Textnummer="+
                        String.valueOf(details.getShownIndex()));

                details = DetailsFragment.newInstance(index);
                FragmentTransaction ft = fm.beginTransaction();
                ft.setCustomAnimations(android.R.animator.fade_in, android.R.animator.fade_out);
//                ft.setTransition(FragmentTransaction.TRANSIT_FRAGMENT_FADE);
                ft.replace(R.id.details, details);
                if (!uselessStackState)
                    ft.addToBackStack(null);
                ft.commit();
            }
        } else {
            Intent intent = new Intent();
            intent.setClass(getActivity(), DetailsActivity.class);
            intent.putExtra("index", index);
            startActivity(intent);
        }
        uselessStackState = false;
    }
}
