package de.zimkand.primedetection;

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Color;
import android.os.AsyncTask;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.support.v7.app.AppCompatActivity;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ProgressBar;
import android.widget.TextView;

public class PrimeActivity extends AppCompatActivity implements OnClickListener {
	
	private final String TAG = this.getClass().getSimpleName();
	private EditText candidate;
	private TextView tvResult, tvMax;
	private Button button;
	private AsyncPrimeTask apt;
	private ProgressBar progressBar;

	private final String MAX_PRIME_STORE = "maxPrimeStore"; 
	private final String MAX_PRIME = "maxPrime";
	private final String BG_COLOR = "bgColor";
	private final String DEF_BG_COLOR = "#FFFFFF";
	private long maxPrime;
	private SharedPreferences spMaxPrime, spDefaultPrefs;

	private class AsyncPrimeTask extends AsyncTask<String, Integer, String> {
	
		private Context context = PrimeActivity.this;
		@Override
		protected void onPreExecute() {
			tvResult.setText("");
			button.setEnabled(false);
			progressBar.setVisibility(ProgressBar.VISIBLE);
			progressBar.setProgress(0);
		}

		@Override
		protected String doInBackground(String... arg) {
			boolean df = false;
			long cand, i, mdc, widthSeg;
			int nSeg = 100;
			if (arg[0].length() == 0)
				return null;
			cand = Long.parseLong(arg[0]);
			mdc = (int) (Math.sqrt(cand)+0.5);
			widthSeg = Math.max(10_000, mdc/nSeg);
			for (i = 2; i <= mdc; i++) {
				if (this.isCancelled())
					return null;
				if (i%widthSeg == 0)
					publishProgress((int)(i/(float)mdc*100));
				if (cand % i == 0) {
					df = true;
					break;
				}
			}
			if (cand <= 1)
				return arg[0] + " " + context.getString(R.string.falseArgument);
			else
			if (df)
				return arg[0] + " " + context.getString(R.string.noPrime) + "\n(" +
						context.getString(R.string.divisor) + Long.toString(i)+ ")";
			else {
				String result = arg[0] + " " + context.getString(R.string.prime);;
				if (cand > maxPrime) {
					SharedPreferences.Editor editor = spMaxPrime.edit();
					editor.putLong(MAX_PRIME, cand);
//					editor.commit();
					editor.apply();
					maxPrime = cand;
					result = result + "\n(" + context.getString(R.string.congratulation) + ")";
				}
				return result;
			}
		}

		@Override
		protected void onProgressUpdate(Integer... progress) {
			progressBar.setProgress(progress[0]);
		}

		@Override
		protected void onPostExecute(String result) {
			tvResult.setText(result);
			progressBar.setVisibility(ProgressBar.GONE);
			if (result != null) {
				tvResult.setText(result);
				if (result.indexOf(getString(R.string.congratulation)) >= 0)
					tvMax.setText(getString(R.string.max) + candidate.getText().toString());
			}
			button.setEnabled(true);
		}
	}
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_prime);

		button = findViewById(R.id.button);
		button.setOnClickListener(this);
		candidate = findViewById(R.id.candidate);
		tvResult = findViewById(R.id.result);
		tvMax = findViewById(R.id.max);
		progressBar = findViewById(R.id.progressBar);

		// Einmalig für den Benutzer Voreinstellungswerte aus renamed_preferencesferences.xml übernehmen
		PreferenceManager.setDefaultValues(this, R.xml.preferences, false);
//		PreferenceManager.setDefaultValues(this, SettingsActivity.PREFILE, MODE_PRIVATE, R.xml.preferences, false);

		spMaxPrime = getSharedPreferences(MAX_PRIME_STORE, MODE_PRIVATE);
		spDefaultPrefs = PreferenceManager.getDefaultSharedPreferences(this);
		//		spDefaultPrefs = this.getSharedPreferences(SettingsActivity.PREFILE, MODE_PRIVATE);
		maxPrime = spMaxPrime.getLong(MAX_PRIME, 0);
		if (maxPrime > 0)
			tvMax.setText(getString(R.string.max) + " " + String.valueOf(maxPrime));
		else
			tvMax.setText("");
	}
	
	@Override
	protected void onResume() {
		super.onResume();
		View root = ((ViewGroup) findViewById(android.R.id.content)).getChildAt(0);
		String s = spDefaultPrefs.getString(BG_COLOR, DEF_BG_COLOR);
		root.setBackgroundColor(Color.parseColor(s));
		// Sehr große Primzahl automatisch als Aufgabe stellen
//		candidate.setText("981168724994134051");
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.menu_main, menu);
		return true;
	}
	
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
			case R.id.action_settings:
				Intent settIntent = new Intent(this, SettingsActivity.class);
				startActivity(settIntent);
				return true;
			default:
				return super.onOptionsItemSelected(item);
		}
	}
	
	@Override
	public void onClick(View arg) {
		button.setEnabled(false);
		apt = new AsyncPrimeTask();
		apt.execute(candidate.getText().toString());
	}
	
	@Override
	protected void onDestroy() {
		super.onDestroy();
		if (apt != null)
			apt.cancel(true);
	}
}
