package de.zimkand.primedetection;

import android.content.BroadcastReceiver;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.ServiceConnection;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.support.v4.content.LocalBroadcastManager;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ProgressBar;
import android.widget.TextView;

import java.lang.ref.WeakReference;

public class PrimeActivity extends AppCompatActivity implements View.OnClickListener {
    private EditText candidate;
    private TextView result;
    private Button button;
    private ProgressBar progressBar;
    private static boolean bound = false;
    private static PrimeServiceBound primeService;
    private static MyServiceConnection serviceConnection;
    private PrimeEvalReceiver receiver;
    public static final String BROADCAST_ACTION = "de.zimkand.primedetection.action.BROADCAST_ACTION";

//    // 8 MByte Speicherverschwendung
//    private long[] memLoad = new long[1048576];
//    static String TAG = "PrimeActivity";

    private class PrimeEvalReceiver extends BroadcastReceiver {
        @Override
        public void onReceive(Context context, Intent intent) {
            Bundle bundle = intent.getExtras();
            float progress = bundle.getFloat("progress");
            if (progress != 0.0f) {
                button.setEnabled(false);
                progressBar.setVisibility(ProgressBar.VISIBLE);
                progressBar.setProgress((int)progress);
                return;
            }
            String ergebnis = bundle.getString("result");
            result.setText(ergebnis);
            button.setEnabled(true);
            progressBar.setVisibility(ProgressBar.GONE);
        }
    }

    private static class MyServiceConnection implements ServiceConnection {
        private WeakReference<PrimeActivity> wRef;

        public MyServiceConnection(PrimeActivity target) {
            wRef = new WeakReference<>(target);
        }

        @Override
        public void onServiceConnected(ComponentName className, IBinder service) {
            PrimeServiceBound.PrimeBinder binder = (PrimeServiceBound.PrimeBinder) service;
            PrimeActivity pa = wRef.get();
            if (pa != null)
                pa.onServiceConnected(binder);
        }

        @Override
        public void onServiceDisconnected(ComponentName component) {
            PrimeActivity pa = wRef.get();
            if (pa != null)
                pa.onServiceDisconnected();
        }
    }


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_prime);
        button = findViewById(R.id.button);
        button.setOnClickListener(this);
        candidate = findViewById(R.id.candidate);
        result = findViewById(R.id.result);
        progressBar = findViewById(R.id.progressBar);

        // An Service binden
        serviceConnection = new MyServiceConnection(this);
        Intent intent = new Intent(this, PrimeServiceBound.class);
        getApplicationContext().bindService(intent, serviceConnection, Context.BIND_AUTO_CREATE);

        // Broadcast Receiver registrieren
        receiver = new PrimeEvalReceiver();
        IntentFilter intentFilter =
                new IntentFilter(BROADCAST_ACTION);
        LocalBroadcastManager.getInstance(this).registerReceiver(receiver, intentFilter);

//        // Anzahl der Threads und Speicherverbrauch im logcat-Fenster protokollieren
//        Runtime runtime = Runtime.getRuntime();
//        int numberOfThreads = Thread.getAllStackTraces().keySet().size();
//        long consumed = (runtime.totalMemory() - runtime.freeMemory())/1048576;
//        Log.i(TAG, "Number of Threads = " + String.valueOf(numberOfThreads) +
//                " Used Memory (MB) = " + String.valueOf(consumed));
    }

//    // Sehr große Primzahl automatisch als Aufgabe stellen
//    @Override
//    protected void onResume() {
//        super.onResume();
//        candidate.setText("981168724994134051");
//    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (!isChangingConfigurations()) {
            getApplicationContext().unbindService(serviceConnection);
        }
        LocalBroadcastManager.getInstance(this).unregisterReceiver(receiver);
    }

    @Override
    public void onClick(View view) {
        if (primeService.isRunning()) {
            button.setEnabled(false);
            return;
        }
        String s = candidate.getText().toString();
        if (s.length() != 0  && bound) {
            result.setText("");
            button.setEnabled(false);
            progressBar.setProgress(0);
            progressBar.setVisibility(ProgressBar.VISIBLE);
            primeService.evalCandidate(s);
        }
    }
    private void onServiceConnected(PrimeServiceBound.PrimeBinder binder) {
        primeService = binder.getService();
        bound = true;
        if (primeService.isRunning())
            button.setEnabled(false);
    }

    private void onServiceDisconnected() {
        bound = false;
    }
}
