package de.zimkand.primedetection;

import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ProgressBar;
import android.widget.TextView;
import java.lang.ref.WeakReference;

public class PrimeActivity extends AppCompatActivity implements View.OnClickListener {
    private EditText candidate;
    private TextView result;
    private Button button;
    private ProgressBar progressBar;
    private boolean bound = false;
    private MyHandler handler;
    private PrimeServiceBound primeService;
    private MyServiceConnection serviceConnection;

//    // 8 MByte Speicherverschwendung
//    private long[] memLoad = new long[1048576];
//    static String TAG = "PrimeActivity";

    private static class MyHandler extends Handler {
        private WeakReference<PrimeActivity> wRef;

        public MyHandler(PrimeActivity target) {
            wRef = new WeakReference<>(target);
        }

        @Override
        public void handleMessage(Message message) {
            PrimeActivity pa = wRef.get();
            if (pa != null)
                pa.updateUI(message);
        }
    }

    private class MyServiceConnection implements ServiceConnection {
        @Override
        public void onServiceConnected(ComponentName className, IBinder service) {
            PrimeServiceBound.PrimeBinder binder = (PrimeServiceBound.PrimeBinder) service;
            primeService = binder.getService();
            bound = true;
            if (primeService.isRunning()) {
                button.setEnabled(false);
                primeService.setHandler(handler);
            }
        }

        @Override
        public void onServiceDisconnected(ComponentName component) {
            bound = false;
        }
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_prime);
        button = findViewById(R.id.button);
        button.setOnClickListener(this);
        candidate = findViewById(R.id.candidate);
        result = findViewById(R.id.result);
        progressBar = findViewById(R.id.progressBar);

        handler = new MyHandler(this);
        serviceConnection = new MyServiceConnection();
        Intent intent = new Intent(this, PrimeServiceBound.class);
        bindService(intent, serviceConnection, Context.BIND_AUTO_CREATE);
        //        getApplicationContext().bindService(intent, serviceConnection, Context.BIND_AUTO_CREATE);

        //        // Anzahl der Threads und Speicherverbrauch im logcat-Fenster protokollieren
        //        Runtime runtime = Runtime.getRuntime();
        //        int numberOfThreads = Thread.getAllStackTraces().keySet().size();
        //        long consumed = (runtime.totalMemory() - runtime.freeMemory())/1048576;
        //        Log.i(TAG, "Number of Threads = " + String.valueOf(numberOfThreads) +
        //                " Used Memory (MB) = " + String.valueOf(consumed));
    }

//    // Sehr große Primzahl automatisch als Aufgabe stellen
//    @Override
//    protected void onResume() {
//        super.onResume();
//        candidate.setText("981168724994134051");
//    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (!isChangingConfigurations()) {
//            getApplicationContext().unbindService(serviceConnection);
            unbindService(serviceConnection);
        }
    }

    @Override
    public void onClick(View view) {
        if (primeService.isRunning()) {
            button.setEnabled(false);
            return;
        }
        String s = candidate.getText().toString();
        if (s.length() != 0  && bound) {
            result.setText("");
            button.setEnabled(false);
            progressBar.setProgress(0);
            progressBar.setVisibility(ProgressBar.VISIBLE);
            primeService.setHandler(handler);
            primeService.evalCandidate(s);
        }
    }

    private void updateUI(Message message) {
        Bundle bundle = message.getData();
        float progress = bundle.getFloat("progress");
        if (progress != 0.0f) {
            progressBar.setVisibility(ProgressBar.VISIBLE);
            progressBar.setProgress((int)progress);
        } else {
            String ergebnis = bundle.getString("result");
            result.setText(ergebnis);
            button.setEnabled(true);
            progressBar.setVisibility(ProgressBar.GONE);
        }
    }
}